#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from builtins import str
from builtins import hex
import sys
from qgis.PyQt.QtCore import QFileInfo, QObject, pyqtSignal
from qgis.core import QgsApplication

# from PyQt4.QtGui import
sys.path.append(QFileInfo(QgsApplication.qgisSettingsDirPath()).path() + "/python/plugins/mapstream")
import serial
import bluetooth
from serial.tools.list_ports import comports

class SerialThread(QObject):
    thread2gui_signal = pyqtSignal(str, name='thread2gui_signal')

    def __init__(self, port, *args, **kwargs):
        QObject.__init__(self, *args, **kwargs)
        self.port = port
        self.__active = False

    def stop(self):
        self.__active = False

    def run(self):
        self.__active = True
        seq = []
        kwargs = {
            'port': self.port,
            'baudrate': 9600,
            'parity': serial.PARITY_NONE,
            'stopbits': serial.STOPBITS_ONE,
            'bytesize': serial.EIGHTBITS,
            'timeout': 0,
        }
        ser = serial.Serial(**kwargs)

        # Comando di richiesta dati: $PLTIT,RQ,HV
        # Risposta di ritorno:       $PLTIT,HV,HDvalue,units,AZvalue,units,INCvalue,units,SDvalue,units,*csum
        # Dove
        #   HD:     distanza orrizontale calcolata in metri (HORIZONTAL DISTANCE)
        #   AZ:     azimuth misurato in gradi (AZIMUTH)
        #   INC:    inclinazione misurata in gradi (INCLINATION)
        #   SD:     distanza inclinata misurata in metri (SLOPE DISTANCE)

        while self.__active:
            # try:  TODO: ripristinare a dev concluso
            if True:
                for c in ser.read():
                    c = chr(c)  # WARNING: improve checking if ascii or not
                    if c in ('\n', ord('\n')):
                        ser.flushInput()
                        x = joined_seq
                        if len(x.split('*')) > 1:
                            # come prima cosa recupero il checksum che mi passa il dispositivo
                            chsun = x.split('*')[1]
                            x = x.split('*')[0]
                            if x[0:1] == "$":
                                # Se la stringa inizia con un $ allora la sintassi è corretta e quindi procedo
                                x = x.split('$')[1]
                                calc_cksum = 0
                                for s in x:
                                    calc_cksum ^= ord(s)
                                # la trasformo in stringa e tolgo l'indicatore di hex
                                checksum1 = str(hex(calc_cksum)).replace("0x", "").upper()
                                if chsun.upper() == checksum1:
                                    # Il checksun calcolato e quello passato dal dispositivo coincido e dunque il dato è buono
                                    signal_cnt = "{}@{}@{}".format(x.split(',')[8], x.split(',')[4], x.split(',')[6])
                                    self.thread2gui_signal.emit(signal_cnt)
                        seq = []
                        break
                    else:
                        if c not in ('\r', ord('\r')):
                            seq.append(c)
                            joined_seq = ''.join(str(v) for v in seq)
            # except Exception as ex:
            #     ser.close()
        ser.close()


def scan_services():

    devices = bluetooth.discover_devices(lookup_names=True)
    _all_ports = list(comports())

    services, lookups = list(), dict()
    for (addr, name) in devices:
        _addr = addr.replace(':', '')
        ports_info = list(filter(lambda info: _addr in info[2], _all_ports))
        if ports_info:
            for port, desc, _ in ports_info:
                label = "{} - {}".format(name, desc)
                services.append(label)
                lookups[label] = port

    return services, lookups
